/***************************************************************************
 *   Copyright (C) 2007 by David Torres   *
 *   aiken669@gmail.com   *
 ***************************************************************************/
#include "omweso.h"
#include "ow_cell.h"


/**
* Creates a new 'ow_cell' object.
* @param screenx Xpos of the cell in the draw_gfx screen.
* @param screeny Ypos of the cell in the draw_gfx screen.
* @param boardside Side of the board (BRD_SIDE_NORTH, BRD_SIDE_SOUTH).
* @param cellx Xpos of the specified cell in the board bcells array.
* @param celly Ypos of the specified cell in the board bcells array.
* @return New 'ow_cell' object.
*/
ow_cell::ow_cell(int screenx, int screeny, int boardside, int cellx, int celly) {
	this->x = screenx;
	this->y = screeny;
  this->cellstatus = CELL_STATUS_NORM;
  this->cellaction = CELL_ACTION_NONE;
	this->cellseeds = 0;
	this->statusaction_timeout = -1;
	this->background = NULL;
}

/**
* Deletes the 'ow_cell' object.
*/
ow_cell::~ow_cell() {
}

/**
* Sets the gfx data in SDL format for the draw_gfx method
* @param gfxdat SDL surface containing the graphics for the cell.
*/
void ow_cell::set_gfxdata(SDL_Surface *gfxdat) {
	this->gfxdata = gfxdat;
}

/**
* Draw the cell in the surface provided
* @param surface SDL surface where to draw the cell, usually is the screen surface.
* @param font SDL_ttf font used to print out cellname.
*/
void ow_cell::draw(SDL_Surface *surface, TTF_Font *font) {
  SDL_Rect src, dest;

  //Draw Background
	if (this->background==NULL) {
		SDL_Color color={0,0,0}; SDL_Surface *tmp;
		SDL_Surface *text_surface;
  	src.x = 0; src.y = 0; src.w = CELL_WIDTH; src.h = CELL_HEIGHT;
		tmp=SDL_CreateRGBSurface(SDL_SWSURFACE,src.w,src.h,8,0,0,0,0);
		this->background = SDL_DisplayFormat(tmp); SDL_FreeSurface(tmp);
  	SDL_BlitSurface(this->gfxdata, &src, this->background, NULL);
		//Draw the cell name
		dest.x=5; dest.y=0;text_surface=TTF_RenderText_Blended(font,this->cellname.c_str(),color);
		SDL_BlitSurface(text_surface,NULL,this->background,&dest);
		SDL_FreeSurface(text_surface);
	}
	dest.x = this->x; dest.y = this->y;
	SDL_BlitSurface(this->background, NULL, surface, &dest);
  
	//Draw cell contents
	src.x = (this->cellseeds*(CELL_WIDTH-CELL_BORDER))+CELL_BORDER; 
	src.y = 32; dest.x = this->x+4; dest.y = this->y+32;
	src.w = 50; src.h = 42;
	SDL_BlitSurface(this->gfxdata, &src, surface, &dest);

	//Draw cell status
	if (this->cellstatus!=CELL_STATUS_NORM) {
  	src.x = (this->cellstatus*(CELL_WIDTH-CELL_BORDER))+CELL_BORDER; 
  	src.y = 4; dest.x = this->x+4; dest.y = this->y+4;
  	src.w = 50; src.h = 24;
  	SDL_BlitSurface(this->gfxdata, &src, surface, &dest);
	}

	//Draw cell action
	if (this->cellaction!=CELL_ACTION_NONE) {
  	src.x = (this->cellaction*(CELL_WIDTH-CELL_BORDER))+CELL_BORDER; 
  	src.y = 4; dest.x = this->x+4; dest.y = this->y+4;
  	src.w = 50; src.h = 24;
  	SDL_BlitSurface(this->gfxdata, &src, surface, &dest);
	}
}

/**
* Count the number of seeds in the cell.
* @return Number of seeds.
*/
int ow_cell::countseeds() {
	return this->cellseeds;
}

/**
* Remove all the seeds from the cell.
*/
void ow_cell::emptyseeds() {
	this->cellseeds = 0;
}

/**
* Add seeds to the cell.
* @param seeds Number of seeds to add to the cell.
* @return Count of seeds in the cell after adding the requested cells.
*/
int ow_cell::addseeds(int seeds) {
	this->cellseeds+=seeds;
	return this->cellseeds;
}

/**
* Take/Remove seeds to the cell.
* @param seeds Number of seeds to remove/take from the cell.
* @return Count of seeds in the cell after taking/removing the requested cells.
*/
int ow_cell::takeseeds(int seeds) {
	this->cellseeds-=seeds;
	return this->cellseeds;
}

/**
* Change the 'action' the cell is performing.
* @param action Action currently running in the cell (CELL_ACTION_CANCEL,CELL_ACTION_MOVEL,CELL_ACTION_MOVER,CELL_ACTION_SELECT,CELL_ACTION_NONE).
* @param timeout program loops the cell holds the action before returning to NONE action. -1 if lasts forever.
*/
void ow_cell::setaction(int action, int timeout) {
	this->cellaction = action;
	this->statusaction_timeout = timeout;
}

/**
* Change the 'status' of the cell.
* @param status Status of the cell (CELL_STATUS_NORM,CELL_STATUS_GREEN,CELL_STATUS_RED). RED is used for negative status like 'illegal movement' or 'captured cell'. GREEN is used for positive status like 'moving'.
* @param timeout program loops the cell holds the status before returning to NORMAL status. -1 if lasts forever.
*/
void ow_cell::setstatus(int status, int timeout) {
	this->cellstatus = status;
	this->statusaction_timeout = timeout;
}

/**
* Update the status and action of the cell
* Basically it is used to update cells who has a timeout defined for setstatus() and setaction().
*/
void ow_cell::update_status_action() {
	if (this->statusaction_timeout==0) {
		this->statusaction_timeout = -1;
		this->cellstatus = CELL_STATUS_NORM;
		this->cellaction = CELL_ACTION_NONE;
	} else {
		if (this->statusaction_timeout>0) {this->statusaction_timeout--;}
	}
}

/**
* Returns a cellXY struct with the cell screen coordinates.
* @return cell x,y coordinates struct.
*/
cellXY ow_cell::getcellcoordXY() {
	cellXY cell_info;
	cell_info.x = this->x;
	cell_info.y = this->y;
	return cell_info;
}

/**
* Set the cell name
* @param cellname The new name to be given to the cell
*/
void ow_cell::setcellname(string newcellname) {
 this->cellname = newcellname;
}

/**
* Checks if (coordx,coordy) point position is inside the cell drawing area in the screen.
* @param coordx Xcoordinate of the point to check.
* @param coordy Ycoordinate of the point to check.
* @return 1 if point is inside cell, 0 if point is outside cell.
*/
int ow_cell::isOverCoordXY(int coordx, int coordy) {
	if ((coordx>=this->x)&&(coordx<=(this->x+CELL_WIDTH))&&
			(coordy>=this->y)&&(coordy<=(this->y+CELL_HEIGHT)))
		return 1;
	return 0;
}
